subroutine MCTrans(xyz,moveatom,Dim,nAt,sigma,L,overlap)
    implicit none
    integer, intent(in) :: moveatom, Dim, nAt
    real(8), intent(in), dimension(0:nAt-1, 0:Dim-1) :: xyz
    real(8), intent(in) :: L, sigma !box length and sphere diameter
    integer, intent(out) :: overlap !1 if sphere overlaps, 0 otherwise
    integer :: i
    real(8), dimension(0:Dim-1) :: rij, xyzi
    real(8) :: r2, sigma2
    sigma2=sigma**2d0
    overlap=0
    xyzi = xyz(moveatom,:)
    do i=0,nAt-1
        rij = xyz(i,:) - xyzi
        rij=rij-L*dnint(rij/L)
        r2=sum(rij**2d0)
        if(r2.le.sigma2) then
          if(i.ne.moveatom) then
            overlap=1
              exit
          endif
        endif
    enddo

return
end

subroutine MCEnergy(xyz,Dim,nAt,sigma,L,overlap)
    implicit none
    integer, intent(in) :: Dim, nAt
    real(8), intent(in), dimension(0:nAt-1, 0:Dim-1) :: xyz
    real(8), intent(in) :: L, sigma !box length and sphere diameter
    integer, intent(out) :: overlap !1 if sphere overlaps, 0 otherwise
    integer :: i,j
    real(8), dimension(0:Dim-1) :: rij, xyzi
    real(8) :: r2, sigma2
    
    sigma2=sigma**2d0
    overlap=0
    do i=0,nAt-1
        xyzi = xyz(i,:)
        do j=i+1,nAt-1
            rij = xyz(j,:) - xyzi
            rij=rij-L*dnint(rij/L)
            r2=sum(rij**2d0)
            if(r2.le.sigma2) then
              overlap=1
              exit
            endif
        enddo
        if(overlap.eq.1) exit
    enddo
    

return
end

subroutine MCVol(xyz,Dim,nAt,sigma,L1,L2,overlap)
    implicit none
    integer, intent(in) :: Dim, nAt
    real(8), intent(inout), dimension(0:nAt-1, 0:Dim-1) :: xyz
    real(8), intent(in) :: sigma !sphere diameter
    real(8), intent(in) :: L1, L2 !box length before and after expansion
    integer, intent(out) :: overlap !1 if sphere overlaps, 0 otherwise
    integer :: i,j
    real(8), dimension(0:Dim-1) :: rij, xyzi
    real(8) :: r2, sigma2
    
!f2py intent(in, out, inplace) :: xyz

    sigma2=sigma**2d0
    overlap=0
    xyz=xyz*(L2/L1)
    do i=0,nAt-1
        xyzi = xyz(i,:)
        do j=i+1,nAt-1
            rij = xyz(j,:) - xyzi
            rij=rij-L2*dnint(rij/L2)
            r2=sum(rij**2d0)
            if(r2.le.sigma2) then
              overlap=1
              exit
            endif
        enddo
        if(overlap.eq.1) exit
    enddo

return 
end

subroutine clustersize(xyz,Dim,nAt,sigma,L,qbond2,solidbond,clusterbond2,bigcluster,clusterdist,clusterlist)
    implicit none
    integer, intent(in) :: Dim, nAt
    real(8), intent(in), dimension(0:nAt-1, 0:Dim-1) :: xyz !Position array
    real(8), intent(in) :: L, sigma !box length and sphere diameter
    real(8), intent(in)  :: qbond2,solidbond,clusterbond2
    integer, intent(out) :: bigcluster,clusterdist(0:nAt),clusterlist(0:nAt-1)
    real(8)  :: q6m(0:nAt-1,0:1,-6:6)
    integer  :: ibond(0:nAt-1)
    
    ! calculate q6m vector for each atoms
    call findq6m(xyz,Dim,nAt,sigma,L,qbond2,q6m)
    !find sij scalar from q6m for each combination of atoms sij=q6m(i) dot q6m(j)
    call findsij(xyz,L,dim,nAt,qbond2,solidbond,q6m,ibond)
    !find distribution of solid like clusters 
    call findcluster(xyz,L,dim,nAt,ibond,clusterbond2,bigcluster,clusterdist,clusterlist)

return
end

subroutine findq6m(xyz,Dim,nAt,sigma,L,qbond2,q6m)
    implicit none
    integer, intent(in) :: Dim, nAt
    real(8), intent(in), dimension(0:nAt-1, 0:Dim-1) :: xyz
    real(8), intent(in) :: L, sigma !box length and sphere diameter
    real(8), intent(in) :: qbond2
    real(8), intent(out) :: q6m(0:nAt-1,0:1,-6:6)
    integer :: i,j,k
    real(8) :: r2,pi,Nbonds,phi,r
    real(8), dimension(0:Dim-1) :: rij,xyzi
    real(8), dimension(-6:6) :: Y6mfactor,Y6m
    real(8), dimension(1:6) :: ct,st



    pi=dacos(-1.0d0)
    !spherical harmonic prefactors
    Y6mfactor(-6)=(1.0d0/64.0d0)*dsqrt(3003.0d0/pi)
    Y6mfactor(-5)=(3.0d0/32.0d0)*dsqrt(1001.0d0/pi)
    Y6mfactor(-4)=(3.0d0/32.0d0)*dsqrt(91.0d0/(2.0d0*pi))
    Y6mfactor(-3)=(1.0d0/32.0d0)*dsqrt(1365.0d0/pi)
    Y6mfactor(-2)=(1.0d0/64.0d0)*dsqrt(1365.0d0/pi)
    Y6mfactor(-1)=(1.0d0/16.0d0)*dsqrt(273.0d0/(2.0d0*pi))
    Y6mfactor(0)=(1.0d0/32.0d0)*dsqrt(13.0d0/pi)
    Y6mfactor(1)=-Y6mfactor(-1)
    Y6mfactor(2)=Y6mfactor(-2)
    Y6mfactor(3)=-Y6mfactor(-3)
    Y6mfactor(4)=Y6mfactor(-4)
    Y6mfactor(5)=-Y6mfactor(-5)
    Y6mfactor(6)=Y6mfactor(-6)
    q6m=0
    do i=0,nAt-1
      xyzi=xyz(i,:)
      Nbonds=0
      Y6m=0
      do j=0,nAt-1
        rij = xyz(j,:) - xyzi
        rij=rij-L*dnint(rij/L)
        r2=sum(rij**2d0)
        if(r2.lt.qbond2) then
        if(i.ne.j) then
          Nbonds=Nbonds+1
          r=dsqrt(r2)
          !calculate sine and cosine
          ct(1)=rij(2)/r
          st(1)=dsqrt(1-ct(1)*ct(1))
          do k=2,6
            st(k)=st(k-1)*st(1)
            ct(k)=ct(k-1)*ct(1)
          enddo
          !find phi in range 0 to 2 Pi
          if(abs(rij(0)).lt.1.0d-15) then
            if(abs(rij(1)).lt.1.0d-15) then
              phi=0.0d0
            elseif(rij(1)>0.0d0) then
              phi=pi/2.0d0
            else
              phi=1.5d0*phi
            endif
          else
            phi=datan(rij(0)/rij(1))
            if(rij(0).lt.0.0d0) then
              phi=phi+Pi
            elseif(rij(1).lt.0.0d0) then
              phi=phi+(2.0d0*Pi)
            endif
          endif
          
          Y6m(-6)=st(6)
          Y6m(-5)=st(5)*ct(1)
          Y6m(-4)=st(4)*((11.0*ct(2))-1.0)
          Y6m(-3)=st(3)*(11.0*ct(3))-(3.0*ct(1))
          Y6m(-2)=st(2)*((33.0*ct(4))-(18.0*ct(2))+1.0)
          Y6m(-1)=st(1)*((33.0*ct(5))-(30.0*ct(3))+(5.0*ct(1)))
          Y6m(0)=(231.0*ct(6))-(315.0*ct(4))+(105.0*ct(2))-5.0
          do k=1,6
            Y6m(k)=Y6m(-k)
          enddo
          !computer q6m
          do k=-6,6
          !real components
            q6m(i,0,k)=q6m(i,0,k)+(Y6m(k)*Y6mfactor(k)*dcos(dble(k)*phi))
            !write(*,*) i,k,q6m(i,0,k),phi,acos(ct(1))
          !imaginary components
            q6m(i,1,k)=q6m(i,1,k)+Y6m(k)*Y6mfactor(k)*dsin(dble(k)*phi)
            !write(*,*) i,k,q6m(i,1,k),phi,acos(ct(1))
          enddo

        endif
        endif
      enddo
      !should be normalized as below, done to match Auer and Frenkel 2004 :(
      !q6m(i,0:1,-6:6)= q6m(i,0:1,-6:6)/Nbonds
    enddo
return
end


subroutine findsij(xyz,L,dim,nAt,qbond2,solidbond,q6m,ibond)
    implicit none
    integer, intent(in) :: nAt,dim
    real(8), intent(in) :: q6m(0:nAt-1,0:1,-6:6) 
    real(8), intent(in) :: solidbond,qbond2
    real(8) :: sij,rij(0:2),xyzi(0:2),r2
    integer :: i,j
    real(8), intent(in), dimension(0:nAt-1, 0:Dim-1) :: xyz
    real(8), intent(in) :: L
    integer, intent(out) :: ibond(0:nAt-1)
    
    sij=0
    ibond=0
    do i=0,nAt-1
      xyzi=xyz(i,:)
      do j=i+1,nAt-1
        rij = xyz(j,:) - xyzi
        rij=rij-L*dnint(rij/L)
        r2=sum(rij**2d0)
        if(r2.lt.qbond2) then
        !isij=0
        sij=sum(q6m(i,0,-6:6)*q6m(j,0,-6:6))+sum(q6m(i,1,-6:6)*q6m(j,1,-6:6))
        !isij=-sum(q6m(i,0,-6:6)*q6m(j,1,-6:6))
        !isij=isij+sum(q6m(i,1,-6:6)*q6m(j,0,-6:6))
        !if(i.eq.j) then
        !write(*,*) i,j,sij(i,j)
        !endif
          if(sij.gt.solidbond) then
            ibond(i)=ibond(i)+1
            ibond(j)=ibond(j)+1
          endif
        endif
      enddo
    enddo

    
    
return
end

subroutine findcluster(xyz,L,dim,nAt,ibond,clusterbond2,bigcluster,clusterdist,clusterlist)
    implicit none
    integer, intent(in) :: nAt,dim
    integer, intent(in) :: ibond(0:nAt-1)
    real(8), intent(in), dimension(0:nAt-1, 0:Dim-1) :: xyz
    real(8), intent(in) :: L,clusterbond2
    integer, intent(out) :: bigcluster
    integer :: clusterlist(0:nAt-1), oldclusteredge(1:nAt),clusteredge(1:nAt)
    integer :: ncluster, oldedgecount, edgecount
    real(8) :: rij(0:dim-1),xyzedge(0:dim-1),r2
    integer :: clusterdist(0:nAt)
    integer :: i,j,k,clusterid
    
    
    clusterlist=0
    clusterdist=0
    clusterid=0
    ncluster=0
    bigcluster=0
    do i=0,nAt-1
        if(clusterlist(i).eq.0) then
          if(ibond(i).lt.6) then
            clusterlist(i)=-1
          else
            clusterid=clusterid+1
            oldedgecount=1
            oldclusteredge(oldedgecount)=i
            clusterlist(i)=clusterid
            ncluster=1
            edgecount=0
            clusteredge=0
            do while(oldedgecount.gt.0) 
              do j=1,oldedgecount
                xyzedge=xyz(oldclusteredge(j),:)
                do k=0,nAt-1
                  if(clusterlist(k).eq.0) then
                  if(ibond(k).lt.6) then
                    clusterlist(k)=-1
                  else
                    rij = xyz(k,:) - xyzedge
                    rij=rij-L*dnint(rij/L)
                    r2=sum(rij**2d0)
                    if(r2.lt.clusterbond2) then
                      edgecount=edgecount+1
                      clusteredge(edgecount)=k
                      clusterlist(k)=clusterid
                      ncluster=ncluster+1
                    endif
                  endif
                  endif
                enddo
              enddo
              oldedgecount=edgecount
              oldclusteredge=clusteredge
              edgecount=0
              clusteredge=0
            enddo
            if(ncluster.ge.bigcluster) then
              bigcluster=ncluster
            endif
            clusterdist(ncluster)=clusterdist(ncluster)+1
            ncluster=0
          endif
        endif
    enddo
 
return 
end