#!/usr/bin/python
#Hard Sphere MC code

#import python modules
import numpy as np
import time
import atomwrite
import copy

#import compiled fortran library
import finallib

import sys
import os

#whether or not to use the 3d visualization
UseVisual = False
if UseVisual:
    #import custom visualization library
    import atomvis
# MC translation move
def MCtrans(Pos,Disp,L,acceptmove):
    #select random atom to displace
    MoveAtom=np.random.randint(len(Pos))
    oldPos=Pos[MoveAtom,:].copy()
    #displace atom and check for overlap
    Move=((np.random.rand(3)*2.0)-1)*Disp
    Pos[MoveAtom,:]+=Move
    overlap=finallib.mctrans(Pos, MoveAtom, sigma, L)
    if overlap==0:
        acceptmove+=1
    else:
        Pos[MoveAtom,:]=oldPos
        
    return Pos,acceptmove

#MC volume change move
def MCVol(Pos,VDisp,L,N,BetaP,acceptmovevol):
    Vmove=((np.random.rand()*2.0)-1)*VDisp
    Vold=L**3.0
    Vnew=Vold+Vmove
    Lnew=Vnew**(1.0/3.0)
    Posnew,overlap=finallib.mcvol(Pos,sigma,L,Lnew)
    if overlap==0:
        dum=N*np.log(Vnew/Vold)-(BetaP*Vmove)
        if dum >= 0:
            Pos=Posnew
            L=Lnew
            acceptmovevol+=1
        elif np.random.rand() < np.exp(dum):
            Pos=Posnew
            L=Lnew
            acceptmovevol+=1
    return Pos, L, acceptmovevol

#get coordinates from PDB file written out for equiliberation run    
def ReadPdb(PdbFile,nAt,frame):
    Pos=[]
    count=0
    for line in file(PdbFile,"r").readlines():
        if line.startswith("HETATM"):
            count+=1
            if count > (frame-1)*nAt and count < ((frame*nAt)+1):
                Pos.append([float(line[30:38]),float(line[38:46]),float(line[46:54])])
    return np.array(Pos,float)

#create list of atom names based on the            
def clusterid(clusterlist):
    atomlist=[]
    for i in clusterlist:
        if i > 0:
            atomlist.append("C")
        else:
            atomlist.append("H")
    return atomlist

#diameter of hard sphere
sigma=1.0
#q6m bond cutoff squared
qbond2=1.4**2.0
#sij solid cutoff (higher than solid bond, solid like)
solidbond=20.0
#solid like particles 
clusterbond2=1.5**2.0
BetaP=15.0
#box length 
L=7.9
#number of spheres
N=500
Pos=ReadPdb("start.pdb",N,201)
np.random.seed()
overlap=finallib.mcenergy(Pos,sigma,L)
print overlap
Disp=0.0332
VDisp=0.0615
traj=25000
step=20
windowmin=1
windowmax=10
totalclusterdist=np.zeros((N+1),float)
oldclusterdist=np.zeros((N+1),float)
acceptmove=0
acceptmovevol=0

#initialize output
bigcluster,clusterdist,clusterlist=finallib.clustersize(Pos,sigma,L,qbond2,solidbond,clusterbond2)
atomlist=clusterid(clusterlist)
atomwrite.WritePdb("test.pdb", Pos, clusterlist, AtomNames=atomlist, L=L, First=True)

Posold=copy.copy(Pos)
sys.stdout.flush()        
for i in range(traj):
    for j in range(step):
        for k in range(N):
            if np.random.rand()< 0.998:
                Pos,acceptmove=MCtrans(Pos,Disp,L,acceptmove)
            else:
                Pos,L,acceptmovevol=MCVol(Pos,VDisp,L,N,BetaP,acceptmovevol)
    bigcluster,clusterdist,clusterlist=finallib.clustersize(Pos,sigma,L,qbond2,solidbond,clusterbond2)
    if ((i+1)%50) == 0:
    	print i+1,((N*np.pi/6.0)/(L**3.0)),bigcluster
    sys.stdout.flush()
    if (bigcluster >= windowmin) and (bigcluster <= windowmax):
        Posold=copy.copy(Pos)
        oldclusterdist=copy.copy(clusterdist)
        totalclusterdist[windowmin:windowmax]+=clusterdist[windowmin:windowmax]
    else:
        Pos=copy.copy(Posold)
        totalclusterdist[windowmin:windowmax]+=oldclusterdist[windowmin:windowmax]
    if ((i+1)%1000)==0:
        bigcluster,clusterdist,clusterlist=finallib.clustersize(Pos,sigma,L,qbond2,solidbond,clusterbond2)
        atomlist=clusterid(clusterlist)
        atomwrite.WritePdb("test.pdb", Pos, clusterlist, AtomNames=atomlist, L=L)
print totalclusterdist[windowmin:windowmax]
sys.stdout.flush()
    



	


